#---
# Excerpted from "Agile Web Development with Rails, 3rd Ed.",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/rails3 for more book information.
#---
require 'test_helper'

class UserStoriesTest < ActionController::IntegrationTest
  fixtures :products

  # あるユーザが、オンラインストアのインデックスページに訪れます。
  # ユーザは、商品を選択して、カートに入れます。その後、ユーザは
  # チェックアウトフォームに詳細を入力してチェックアウトします。
  # チェックアウトフォームを送信すると、カートに追加された商品に
  # 対応する1つの品目とともに、ユーザの情報が含まれた注文が
  # 作成されます。
  
  test "buying a product" do
    #START:setup
    LineItem.delete_all
    Order.delete_all
    ruby_book = products(:ruby_book)
    #END:setup

    #START:step1
    get "/store/index"
    assert_response :success
    assert_template "index"
    #END:step1
    
    #START:step2
    xml_http_request :put, "/store/add_to_cart", :id => ruby_book.id
    assert_response :success 
    
    cart = session[:cart]
    assert_equal 1, cart.items.size
    assert_equal ruby_book, cart.items[0].product
    #END:step2
    
    #START:step3
    post "/store/checkout"
    assert_response :success
    assert_template "checkout"
    #END:step3
    
    #START:step4
    post_via_redirect "/store/save_order",
                      :order => { :name     => "Dave Thomas",
                                  :address  => "123 The Street",
                                  :email    => "dave@pragprog.com",
                                  :pay_type => "check" }
    assert_response :success
    assert_template "index"
    assert_equal 0, session[:cart].items.size
    #END:step4
    
    #START:step5
    orders = Order.find(:all)
    assert_equal 1, orders.size
    order = orders[0]
    
    assert_equal "Dave Thomas",       order.name
    assert_equal "123 The Street",    order.address
    assert_equal "dave@pragprog.com", order.email
    assert_equal "check",             order.pay_type
    
    assert_equal 1, order.line_items.size
    line_item = order.line_items[0]
    assert_equal ruby_book, line_item.product
    #END:step5
  end
end