#---
# Excerpted from "Agile Web Development with Rails, 3rd Ed.",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/rails3 for more book information.
#---
# このコントローラに追加されたフィルタはアプリケーションのすべてのコントローラに適用
# されます。同様に、追加されたすべてのメソッドはすべてのコントローラで使用できます。

#START:auth
#START:i18n
class ApplicationController < ActionController::Base
  layout "store"
  before_filter :authorize, :except => :login
  before_filter :set_locale
  #...
#END:i18n
#END:auth
  helper :all # include all helpers, all the time

  # 詳細はActionController::RequestForgeryProtectionを参照。
  # クッキーのセッションストアを使っていない場合は:secretをコメント解除。
  protect_from_forgery :secret => '8fc080370e56e929a2d5afca5540a0f7'
  
  # 詳細はActionController::Baseを参照。
  # 送信された機密データのパラメータの内容をアプリケーションのログからフィルタリングする
  # 場合はコメントを解除（以下では"password"のような名前のフィールドをフィルタリングする）
  # filter_parameter_logging :password
#START:auth

#START:i18n
protected
#END:i18n
  def authorize
    unless User.find_by_id(session[:user_id])
      #START_HIGHLIGHT
      if session[:user_id] != :logged_out
        #START:basic
        authenticate_or_request_with_http_basic('Depot') do |username, password|
          user = User.authenticate(username, password)
          session[:user_id] = user.id if user
        end
        #END:basic
      else
        flash[:notice] = "ログインしてください"
        redirect_to :controller => 'admin', :action => 'login'
      end
      #END_HIGHLIGHT
    end
  end

#START:i18n
  def set_locale
    session[:locale] = params[:locale] if params[:locale]
    I18n.locale = session[:locale] || I18n.default_locale

    locale_path = "#{LOCALES_DIRECTORY}#{I18n.locale}.yml"

    unless I18n.load_path.include? locale_path
      I18n.load_path << locale_path
      I18n.backend.send(:init_translations)
    end

  rescue Exception => err
    logger.error err
    flash.now[:notice] = "#{I18n.locale} には翻訳されていません"

    I18n.load_path -= [locale_path]
    I18n.locale = session[:locale] = I18n.default_locale
  end
end
#END:i18n
#END:auth