#---
# Excerpted from "Agile Web Development with Rails, 3rd Ed.",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/rails3 for more book information.
#---
$: << File.dirname(__FILE__)
require "connect"
require "logger"
require "rubygems"
require "activerecord"

require "vendor/plugins/acts_as_tree/lib/active_record/acts/tree.rb"
require "vendor/plugins/acts_as_tree/init"

#ActiveRecord::Base.logger = Logger.new(STDOUT)

ActiveRecord::Schema.define do

  #START:migrations
  create_table :categories, :force => true do |t|
    t.string  :name
    t.integer :parent_id
  end
  #END:migrations
end




#START:model
class Category < ActiveRecord::Base
  acts_as_tree  :order => "name"
end
#END:model


#START:setup
root        = Category.create(:name => "書籍")
fiction     = root.children.create(:name => "フィクション")
non_fiction = root.children.create(:name => "ノンフィクション")

non_fiction.children.create(:name => "コンピュータ")
non_fiction.children.create(:name => "サイエンス")
non_fiction.children.create(:name => "芸術歴史")

fiction.children.create(:name => "ミステリー")
fiction.children.create(:name => "ロマンス")
fiction.children.create(:name => "SF")
#END:setup

def display_children(order)
  puts order.children.map {|child| child.name }.join(", ")
end

#START:demo
display_children(root)             # フィクション、ノンフィクション

sub_category = root.children.first
puts sub_category.children.size    #=> 3
display_children(sub_category)     #=> ミステリー、ロマンス、SF

non_fiction = root.children.find(:first, :conditions => "name = 'ノンフィクション'")

display_children(non_fiction)      #=> 芸術歴史、コンピュータ、サイエンス
puts non_fiction.parent.name       #=> 書籍
#END:demo