#---
# Excerpted from "Agile Web Development with Rails, 3rd Ed.",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/rails3 for more book information.
#---
$: << File.dirname(__FILE__)
require "connect"
require "logger"

#ActiveRecord::Base.logger = Logger.new(STDOUT)

require "rubygems"
require "activerecord"


ActiveRecord::Schema.define do
  #START:migration
  create_table :products, :force => true do |t|
    t.string :title
    t.text :description
    # ...
    t.integer :line_items_count, :default => 0
  end
  #END:migration
  
  create_table :line_items, :force => true do |t|
    t.integer :product_id
    t.integer :order_id
    t.integer :quantity
    t.decimal :unit_price, :precision => 8, :scale => 2
  end
  
end

class Product < ActiveRecord::Base
  has_many :line_items
end

#START:line_item
class LineItem < ActiveRecord::Base
  belongs_to :product, :counter_cache => true
end
#END:line_item

#START:count_issue
product = Product.create(:title => "Programming Ruby",
                         :description => " ... ")
line_item = LineItem.new
line_item.product = product
line_item.save
puts "メモリ内サイズ = #{product.line_items.size}"                       #=> 0
puts "リフレッシュされたサイズ = #{product.line_items(:refresh).size}"   #=> 1
#END:count_issue



LineItem.delete_all
Product.delete_all

#START:count_fix
product = Product.create(:title => "Programming Ruby", 
                         :description => " ... ")
product.line_items.create
puts "メモリ内サイズ = #{product.line_items.size}"                       #=> 1
puts "リフレッシュされたサイズ = #{product.line_items(:refresh).size}"   #=> 1
#END:count_fix


