#---
# Excerpted from "Agile Web Development with Rails, 3rd Ed.",
# published by The Pragmatic Bookshelf.
# Copyrights apply to this code. It may not be used to create training material, 
# courses, books, articles, and the like. Contact us if you are in doubt.
# We make no guarantees that this code is fit for any purpose. 
# Visit http://www.pragmaticprogrammer.com/titles/rails3 for more book information.
#---
﻿require 'activesupport'
require 'action_controller'
require 'action_controller/integration'

ActionController::Routing.use_controllers! [ "article", "blog" ]
rs = ActionController::Routing::Routes
app = ActionController::Integration::Session.new

#START:draw
ActionController::Routing::Routes.draw do |map| 
  
  # 'http://my.app/blog/'はインデックスを表示する 
  map.connect "blog/", 
              :controller => "blog", 
              :action => "index" 

  #START:example
  # year、year/month、year/month/dayに対して記事を返す
  map.connect "blog/:year/:month/:day", 
              :controller => "blog", 
              :action => "show_date", 
              :requirements => { :year => /(19|20)\d\d/,
                                 :month => /[01]?\d/, 
                                 :day => /[0-3]?\d/}, 
              :day => nil, 
              :month => nil 

  # idで識別された記事を表示する
  map.connect "blog/show/:id", 
              :controller => "blog", 
              :action => "show", 
              :id => /\d+/ 
              
  # 管理機能用の通常のRailsのルーティング
  map.connect "blog/:controller/:action/:id" 
  #END:example              

  # 不正なリクエストをうまく処理するためにすべてのURLを受け取るルーティング
  map.connect "*anything", 
              :controller => "blog", 
              :action => "unknown_request" 
end
#END:draw

rs.recognize_path "/blog"
rs.recognize_path "/blog/show/123"
rs.recognize_path "/blog/2004"
rs.recognize_path "/blog/2004/12"
rs.recognize_path "/blog/2004/12/25"
rs.recognize_path "/blog/article/edit/123"
rs.recognize_path "/blog/article/show_stats"
rs.recognize_path "/blog/wibble"
rs.recognize_path "/junk"

last_request = rs.recognize_path "/blog/2006/07/28"
rs.generate({:day => 25}, last_request)
rs.generate({:year => 2005}, last_request)
rs.generate({:action => "show" , :id => 123}, last_request)
rs.generate({:year => 2006}, last_request)
rs.generate({:year => 2006, :month => nil}, last_request)

app.url_for :controller => 'blog', :action => 'show_date', :year => 2002
app.url_for :controller => 'blog', :action => 'show_date', :overwrite_params => {:year => "2002" }